#include "window.h"

#include <QtGui/QTreeWidget>
#include <QtGui/QPushButton>
#include <QtGui/QGridLayout>
#include <QtGui/QValidator>
#include <QtCore/QDebug>
#include <KDE/KInputDialog>
#include <KDE/KMessageBox>

Window::Window()
    : QWidget(0)
    , dockManagerInterface(0)
{
    QPushButton *capabilitiesBtn = new QPushButton("Get Capabilities", this);
    QPushButton *itemsBtn = new QPushButton("Get Items", this);
    QPushButton *badgeBtn = new QPushButton("Set Badge", this);
    QPushButton *iconBtn = new QPushButton("Set Icon", this);
    QPushButton *overlayBtn = new QPushButton("Set Overlay", this);
    QGridLayout *layout = new QGridLayout(this);
    treeWidget = new QTreeWidget(this);
    treeWidget->setHeaderLabels(QStringList() << "Path" << "DesktopFile");
    treeWidget->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
    treeWidget->setRootIsDecorated(false);
    layout->addWidget(treeWidget, 0, 0, 6, 1);
    layout->addWidget(capabilitiesBtn, 0, 1);
    layout->addWidget(itemsBtn, 1, 1);
    layout->addWidget(badgeBtn, 2, 1);
    layout->addWidget(iconBtn, 3, 1);
    layout->addWidget(overlayBtn, 4, 1);
    connect(capabilitiesBtn, SIGNAL(clicked(bool)), SLOT(getCapabilities()));
    connect(itemsBtn, SIGNAL(clicked(bool)), SLOT(getItems()));
    connect(badgeBtn, SIGNAL(clicked(bool)), SLOT(setBadge()));
    connect(iconBtn, SIGNAL(clicked(bool)), SLOT(setIcon()));
    connect(overlayBtn, SIGNAL(clicked(bool)), SLOT(setOverlay()));

    watcher = new QDBusServiceWatcher(this);
    watcher->setConnection(QDBusConnection::sessionBus());
    watcher->setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
    watcher->addWatchedService(net::launchpad::DockManager::staticInterfaceName());
    connect(watcher, SIGNAL(serviceOwnerChanged(QString, QString, QString)), this, SLOT(serviceOwnerChanged(QString, QString, QString)));

    QDBusReply<bool> reply = QDBusConnection::sessionBus().interface()->isServiceRegistered(net::launchpad::DockManager::staticInterfaceName());
    if (reply.isValid() && reply.value()) {
        serviceOwnerChanged(net::launchpad::DockManager::staticInterfaceName(), QString(), QLatin1String("X"));
        getItems();
    }
}

void Window::getCapabilities()
{
    if (!dockManagerInterface) {
        return;
    }

    QDBusReply<QStringList> reply = dockManagerInterface->GetCapabilities();

    if (reply.isValid()) {
        KMessageBox::informationList(this, "DockManager supports the following capabilities:", reply.value());
    }
}

void Window::getItems()
{
    if (!dockManagerInterface) {
        return;
    }

    treeWidget->clear();

    QDBusReply<QList<QDBusObjectPath> > reply = dockManagerInterface->GetItems();

    if (reply.isValid()) {
        QList<QDBusObjectPath> paths = reply.value();
        foreach(const QDBusObjectPath &path, paths) {
            net::launchpad::DockItem *item = new net::launchpad::DockItem(net::launchpad::DockManager::staticInterfaceName(), path.path(), QDBusConnection::sessionBus(), this);
            dockItems.insert(path.path(), item);
            new QTreeWidgetItem(treeWidget, QStringList() << path.path() << item->desktopFile());
        }
    }
    treeWidget->resizeColumnToContents(0);
    treeWidget->resizeColumnToContents(1);
}

class Validator : public QValidator
{
public:
    Validator() : QValidator(0) { }
    State validate(QString &, int &) const { return QValidator::Acceptable; }
};

void Window::setBadge()
{
    net::launchpad::DockItem *item = getCurrentItem();

    if (!item) {
        return;
    }

    bool ok;
    Validator valid;
    QString val = KInputDialog::getText("Set DockItem Badge", "Enter new badge:", QString(), &ok, this, &valid);

    if (ok) {
        QVariantMap items;
        items.insert("badge", val);
        item->UpdateDockItem(items);
    }
}

void Window::setIcon()
{
    net::launchpad::DockItem *item = getCurrentItem();

    if (!item) {
        return;
    }

    bool ok;
    Validator valid;
    QString val = KInputDialog::getText("Set DockItem Icon", "Enter new icon (file or name):", QString(), &ok, this, &valid);

    if (ok) {
        QVariantMap items;
        items.insert("icon-file", val);
        item->UpdateDockItem(items);
    }
}

void Window::setOverlay()
{
    net::launchpad::DockItem *item = getCurrentItem();

    if (!item) {
        return;
    }

    bool ok;
    Validator valid;
    QString val = KInputDialog::getText("Set DockItem Overlay", "Enter new overlay (file or name):", QString(), &ok, this, &valid);

    if (ok) {
        QVariantMap items;
        items.insert("x-kde-overlay", val);
        item->UpdateDockItem(items);
    }
}

void Window::serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
{
    Q_UNUSED(name)
    if (newOwner.isEmpty()) {
        dockManagerInterface->deleteLater();
        dockManagerInterface = 0;

        foreach(net::launchpad::DockItem *item, dockItems.values()) {
            item->deleteLater();
            item = 0;
        }
        dockItems.clear();
        treeWidget->clear();
    } else if (oldOwner.isEmpty()) {
        dockManagerInterface = new net::launchpad::DockManager(net::launchpad::DockManager::staticInterfaceName(), "/net/launchpad/DockManager", QDBusConnection::sessionBus(), this);
    }
}

net::launchpad::DockItem * Window::getCurrentItem()
{
    QTreeWidgetItem *item = treeWidget->currentItem();

    if (item && dockItems.contains(item->text(0))) {
        return dockItems[item->text(0)];
    }

    return 0;
}

#include "window.moc"
